﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NHibernate.Cfg;
using NHibernate;
using DataTransfer;
using NHibernate.Expression;

namespace DataAccessLayer
{
    public class NHibernateDataProvider
    {

        #region Fields

        private ISession _session;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the NHibernateDataProvider class.
        /// </summary>
        public NHibernateDataProvider(ISession session)
        {
            _session = session;
        }

        public IList<YoYoCustomer> GetCustomersOrderingYoYosSince(DateTime orderDate)
        {
            return _session.CreateCriteria(typeof(YoYoCustomer))
                .Add(Expression.Gt(YoYoCustomer.MappingNames._orderdate, orderDate))
                .List<YoYoCustomer>();
        }


        #endregion

        #region Properties

        public ISession Session
        {
            set
            {
                _session = value;
            }
        }

        #endregion

        #region Public Methods

        public int AddCustomer(Customer customer)
        {

            using (ITransaction tx = _session.BeginTransaction())
            {
                try
                {
                    int newId = (int)_session.Save(customer);
                    _session.Flush();
                    tx.Commit();
                    return newId;
                }
                catch (NHibernate.HibernateException)
                {
                    tx.Rollback();
                    throw;
                }
            }

        }

        public IList<Customer> CRIT_GetCustomersByFirstname(string firstname)
        {

            return _session.CreateCriteria(typeof(Customer)).Add(new EqExpression(Customer.MappingNames.Firstname, firstname)).List<Customer>();

        }

        public IList<Customer> CRIT_GetCustomersByFirstnameAndLastname(string firstname, string lastname)
        {

            return _session.CreateCriteria(typeof(Customer)).Add(new EqExpression(Customer.MappingNames.Firstname, firstname)).Add(new EqExpression(Customer.MappingNames.Lastname, lastname)).List<Customer>();

        }

        public IList<Customer> CRIT_GetCustomersOrderedByLastnames()
        {

            return _session.CreateCriteria(typeof(Customer)).AddOrder(new NHibernate.Expression.Order(Customer.MappingNames.Lastname,true)).List<Customer>();

        }

        public IList<Customer> CRIT_GetCutomersWithIdGreaterThan(int customerId)
        {

            return _session.CreateCriteria(typeof(Customer)).Add(new GtExpression(Customer.MappingNames.CustomerId, customerId)).List<Customer>();

        }

        public IList<string> CRIT_GetDistinctCustomerFirstnames()
        {

            return _session.CreateCriteria(typeof(Customer)).SetProjection(Projections.Distinct(Projections.Property("Firstname"))).List<string>();

        }

        public IList<Customer> CRIT_GetDistinctCustomersWithOrdersSince(DateTime orderDate)
        {
            IList<int> ids = _session.CreateCriteria(typeof(Customer))
                .SetProjection(Projections.Distinct(Projections.ProjectionList().Add(Projections.Property(Customer.MappingNames.CustomerId))))
                .CreateCriteria(Customer.MappingNames.Orders)
                .Add(Expression.Gt(DataTransfer.Order.MappingNames.OrderDate, orderDate))
                .List<int>();

            return _session.CreateCriteria(typeof(Customer))
                .Add(Expression.In(Customer.MappingNames.CustomerId, ids.ToArray<int>()))
                .List<Customer>();
        }

        public void DeleteCustomer(Customer customer)
        {

            using (ITransaction tx = _session.BeginTransaction())
            {
                try
                {
                    _session.Delete(customer);
                    _session.Flush();
                    tx.Commit();
                }
                catch (NHibernate.HibernateException)
                {
                    tx.Rollback();
                    throw;
                }
            }

        }

        public Customer GetCustomerById(int customerId)
        {

            return _session.Get<Customer>(customerId);

        }

        public Customer GetCustomerByIdAndLeaveSessionIntact(int customerId)
        {

            return _session.Get<Customer>(customerId);
        }

        //public IList<CustomerFirstnameCounter> GetCustomerFirstnameCounts()
        //{

        //    return _session.CreateQuery("select new CustomerFirstnameCounter(c.Name.Firstname, count(c.Name.Firstname)) from Customer c group by c.Name.Firstname").List<CustomerFirstnameCounter>();

        //}

        public IList<Customer> GetCustomersByExample(Customer customerSample)
        {

            return _session.CreateCriteria(typeof(Customer)).Add(Example.Create(customerSample)).List<Customer>();

        }

        public IList<Customer> GetCustomersByFirstname(string firstname)
        {

            return _session.CreateQuery("select from Customer c where c.Firstname=:fn").SetString("fn", firstname).List<Customer>();

        }

        public IList<Customer> GetCustomersByFirstnameAndLastname(string firstname, string lastname)
        {

            return _session.CreateQuery("select from Customer c where c.Firstname=:fn and c.Lastname=:ln").SetString("fn", firstname).SetString("ln", lastname).List<Customer>();

        }

        public IList<Customer> GetCustomersOrderedByLastnames()
        {

            return _session.CreateQuery("select from Customer c order by c.Lastname").List<Customer>();

        }

        public IList<Customer> GetCustomersWithIdGreaterThan(int customerId)
        {

            return _session.CreateQuery("select from Customer c where c.CustomerId > :cid").SetInt32("cid", customerId).List<Customer>();

        }

        public IList<Customer> GetCustomersWithOrdersHavingProduct(int productId)
        {

            return _session.CreateCriteria(typeof(Customer))
                .CreateCriteria("Orders")
                .CreateCriteria("Products")
                .Add(Expression.Eq("ProductId", productId))
                .List<Customer>();

        }

        public IList<string> GetDistinctCustomerFirstnames()
        {

            return _session.CreateQuery("select distinct c.Firstname from Customer c").List<string>();

        }

        public IList<Customer> GetDistinctCustomersWithOrdersSince(DateTime orderDate)
        {
            return _session.CreateQuery("select distinct c from Customer c , c.Orders.elements o where o.OrderDate > :orderDate").SetDateTime("orderDate", orderDate).List<Customer>();
        }

        public void SaveOrUpdateCustomers(IList<Customer> customers)
        {

            using (ITransaction tx = _session.BeginTransaction())
            {
                try
                {
                    foreach (Customer c in customers)
                        _session.SaveOrUpdate(c);
                    _session.Flush();
                    tx.Commit();
                }
                catch (NHibernate.HibernateException)
                {
                    tx.Rollback();
                    throw;
                }
            }

        }

        public void UpdateCustomer(Customer customer)
        {

            using (ITransaction tx = _session.BeginTransaction())
            {
                try
                {
                    _session.Update(customer);
                    _session.Flush();
                    tx.Commit();
                }
                catch (NHibernate.HibernateException)
                {
                    tx.Rollback();
                    throw;
                }
            }

        }

        #endregion

        #region Private Methods

        #endregion

    }
}